#include "app_error.h"
#include "ble_gap.h"
#include "ble_advdata.h"
#include "es_slot.h"

#include "beacon_communication_typedefs.h"
#include "beacon_adv.h"
#include "beacon_acc_statistics.h"

#include "beacon_ibeacon.h"
#include "beacon_tri_frame.h"


#define NRF_LOG_MODULE_NAME beacon_adv
#include "nrf_log.h"
NRF_LOG_MODULE_REGISTER();


/* Private function declarations */

/**
 * Prepare an rms/minmax frame
 *
 * @param p_es_slot is a pointer to the slot containing the frame
 */
static void m_beacon_adv_populate_acc_stats_frame(es_slot_t* p_es_slot);



/**@brief Function for setting advertisement data, using 'ble_advdata_encode'.
 *
 * @param[in] p_scrsp_data      Scan response data.
 * @param[in] p_es_data_array   Eddystone service data array.
 */
static void m_beacon_adv_fill_proprietary_data(uint8_t type, ble_gap_adv_data_t * const p_adv_data);

/* Private function implementations */

static void m_beacon_adv_populate_acc_stats_frame(es_slot_t* p_es_slot)
{
    beacon_acc_statistics_prepare(p_es_slot);
}



static void m_beacon_adv_fill_proprietary_data(uint8_t type, ble_gap_adv_data_t * const p_adv_data)
{
    ble_advdata_t adv_data;
    ret_code_t    err_code;

    static ble_advdata_manuf_data_t manuf_data;


    // Build and set advertising data.
    memset(&adv_data, 0, sizeof(ble_advdata_t));

    adv_data.p_manuf_specific_data = &manuf_data;

    switch(type) {
        case BEACON_FRAME_TYPE_IBEACON:
            adv_data.name_type = BLE_ADVDATA_NO_NAME;
            adv_data.flags = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;
            beacon_ibeacon_get_advertisement_data(&manuf_data);

            break;
        case BEACON_FRAME_TYPE_TRI:
            adv_data.name_type = BLE_ADVDATA_NO_NAME;
         //  adv_data.flags = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;

            /*add for sdk17*/
            beacon_tri_frame_get_advertisement_data(&manuf_data);

            break;

        default:
            NRF_LOG_INFO("Unidentified proprietary frame 0x%x", type);
            return;
            break;
    }

  //  err_code = ble_advdata_set(&adv_data, NULL);  //special for sdk14

	/*add for sdk17*/
    err_code = ble_advdata_encode(&adv_data, p_adv_data->adv_data.p_data, &p_adv_data->adv_data.len);
    APP_ERROR_CHECK(err_code);
    p_adv_data->scan_rsp_data.p_data = NULL;
    p_adv_data->scan_rsp_data.len    = 0;

}


bool beacon_adv_custom_frame_prepare(es_slot_t* p_es_slot, ble_gap_adv_data_t * const p_adv_data)
{
    uint8_t type = p_es_slot->adv_frame.type;
    switch(type) {
        case BEACON_FRAME_TYPE_ACC_STATISTICS:
            m_beacon_adv_populate_acc_stats_frame(p_es_slot);
            return true;
        break;

        case BEACON_FRAME_TYPE_IBEACON:
            m_beacon_adv_fill_proprietary_data(BEACON_FRAME_TYPE_IBEACON, p_adv_data);
            return false;
        case BEACON_FRAME_TYPE_TRI:
            m_beacon_adv_fill_proprietary_data(BEACON_FRAME_TYPE_TRI, p_adv_data);
            return false;

        default:
            NRF_LOG_INFO("Unknown custom frame 0x%x slot %d", p_es_slot->adv_frame.type, p_es_slot->slot_no);
            return false;
        break;
    }

}
